#include <pspdisplay.h>	//screen output?
#include <pspgu.h>
#include "../defines.h"
#include "core.h"
#include "../renderer/graphics.h"
#include <pspaudiolib.h>
#include <pspaudio.h>

//static
int CEngineCore::SetupCallbacks(void)
{
    int thid = sceKernelCreateThread("update_thread", CEngineCore::callbackThread, 0x11, 0xFA0, 0, 0);

    if (thid >= 0) sceKernelStartThread(thid, 0, 0);
        return(0);       
}
   
//static
int CEngineCore::exit_callback(int arg1, int arg2, void * common)
{
    sceKernelExitGame();
    return(0);
}
   
//static
int CEngineCore::callbackThread(SceSize args, void *argp)
{
    int cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
    sceKernelRegisterExitCallback(cbid);

    sceKernelSleepThreadCB();
    return(0);               
} 

//------------------------------------------------------------------------------
// Initialise everything
//------------------------------------------------------------------------------
int CEngineCore::Init(enum PspCtrlMode _ctrlmode)
{
	SetupCallbacks();
	
	initGraphics(); 
//	InitGFX();
	pspDebugScreenInit();
	m_Pad.Init(_ctrlmode);
	m_Timer.Init();
	return 1;
}
//------------------------------------------------------------------------------
// End Game
//------------------------------------------------------------------------------
int CEngineCore::Shutdown()
{
	printf("Shutting Down...");
	sceKernelExitGame();
	return 1;
}


	
int CEngineCore::InitGFX()
{
	return 1;
}



//------------------------------------------------------------------------------
// Pad Input
//------------------------------------------------------------------------------
int CEngineCore::UpdatePad()
{
	return m_Pad.Update();
}


//------------------------------------------------------------------------------
// Update everything
//------------------------------------------------------------------------------

u64 CEngineCore::Update()
{
	
	m_Pad.Update();
	return m_Timer.Update();
}


//------------------------------------------------------------------------------
// Start/end drawing Frame
//------------------------------------------------------------------------------
int CEngineCore::StartFrame()
{
	clearScreen(RGB(0,0,0));
	pspDebugScreenSetXY(0, 0);
	return 1;	
}

int CEngineCore::EndFrame()
{

	flipScreen();
//	sceDisplayWaitVblankStart();
	return 1; 		
}



//==============================================================================
// Control Pad ops
//==============================================================================

int CCtrlPad::Init(enum PspCtrlMode _mode)
{
	sceCtrlSetSamplingCycle(0);
	return sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
}

int CCtrlPad::Update()
{
//	sceCtrlReadBufferPositive(&m_Pad, 1); 	// slower(?)
	if(!sceCtrlPeekBufferPositive(&m_Pad, 1))
		return State;
			
	
	Old = State;
	State = m_Pad.Buttons;
	Toggle = Old ^ State;
	Press = Toggle & State;
	Release =  Toggle & Old;
	
	Lx = m_Pad.Lx;
	Ly = m_Pad.Ly;
	
	return State;
};



//==============================================================================
// Frame counter  (lifted from blit sample)
//==============================================================================
void CTimer::Init()
{
	m_CurrentMS = 1.0f;
	m_LastTick = 0;
	m_TickFreq = sceRtcGetTickResolution();
	m_FrameCount = 0;
}

u64  CTimer::Update()
{
	u64 tp;
	u64 currentTick;
	
	++m_FrameCount;
	sceRtcGetCurrentTick(&currentTick);
	tp = currentTick - m_LastTick;
	if ((tp) >= m_TickFreq)
	{
		float timeSpan = ((int)(currentTick - m_LastTick)) / (float)m_TickFreq;
		m_CurrentMS = timeSpan / m_FrameCount;

		m_FrameCount = 0;
		sceRtcGetCurrentTick(&m_LastTick);
	}
	
	m_FPS = 1.0f / m_CurrentMS;
	
	return tp;
}

float CTimer::GetFPS()
{
	return m_FPS;
}
