#include <pspkernel.h>	//kernel
#include <stdlib.h>		//rand()
#include <stdio.h>		//sprintf
#include <psputils.h>	//time

#include "defines.h" 
#include "core/core.h" 
#include "renderer/graphics.h" 
#include "renderer/text.h" 

#include "globals.h"
#include "menus.h"

//PSP Program info
PSP_MODULE_INFO("PONG", 0, 1, 1);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

//==============================================================================
// DEMO / ATTRACT MODE
//==============================================================================

//------------------------------------------------------------------------------
// Get Input
//------------------------------------------------------------------------------
int InputDemo(void* _arg)
{
	if(g_Core.m_Pad.State != 0) 
	{
		g_pInput = &InputMenu;
		g_pLogic = &LogicMenu;
		g_pRender = &RenderMenu;
	}

	return 1;
}


//------------------------------------------------------------------------------
// Think
//------------------------------------------------------------------------------
int LogicDemo(void* _arg)
{
	// Update	
	P1.Update(Playfield, Ball.m_Pos.y);
	P2.Update(Playfield, Ball.m_Pos.y);
	
	//Waiting for serve?
	switch(g_ToServe)
	{
	case 0:
		Ball.Update(Playfield, P1, P2);
		break;
	case 1:
			{
			Ball.m_Pos.x = P1.m_Pos.x + P1.m_HScale.x + Ball.m_HScale.x + 8;
			Ball.m_Pos.y = P1.m_Pos.y;
			Ball.m_Vel.x = Ball.m_Speeds.x;
			Ball.m_Vel.y = (P1.m_Pos.y / SCREEN_HEIGHT) - 0.5f;
		}
		break;
	case 2:
			{
			Ball.m_Pos.x = P2.m_Pos.x - P1.m_HScale.x - Ball.m_HScale.x - 8;
			Ball.m_Pos.y = P2.m_Pos.y;
			Ball.m_Vel.x = -1 * Ball.m_Speeds.x;
			Ball.m_Vel.y = (P2.m_Pos.y / SCREEN_HEIGHT) - 0.5f;
		}
		break;
	}
	
	
	//Win state
	if((P1.m_Score == WinAt) || (P2.m_Score == WinAt))
	{
		if(P1.m_Score > P2.m_Score)
			P1.m_ScoreGamesWon++;
		else
			P2.m_ScoreGamesWon++;
			
		g_pInput = &InputMenu;
		g_pLogic = &LogicMenu;
		g_pRender = &RenderMenu;
	}
	return 1;		
}
//==============================================================================
// GAME IN PROGRESS FUNCTIONS
//==============================================================================

//------------------------------------------------------------------------------
// Get Everything ready
//------------------------------------------------------------------------------
int Initialise(void* _arg)
{
	g_Core.Init(PSP_CTRL_MODE_ANALOG);

	g_Text.Init("Media/fnt.png");
	NewGame(GT_VS);
	return 1;
}

int NewGame(gametype _type)
{
	Playfield.Init(SCREEN_WIDTH*0.5f, SCREEN_HEIGHT*0.5, SCREEN_WIDTH * 0.5f - 32, SCREEN_HEIGHT*0.5-2);

	switch(_type)
	{
	case GT_SINGLE:
		P1.Init(1, 0);
		P2.Init(2, 0);
		break;
	case GT_VS:
		P1.Init(1, 0);
		P2.Init(2, CPUIntelligence);
		break;
	case GT_DEMO:
		P1.Init(1, CPUIntelligence);
		P2.Init(2, CPUIntelligence);
		break;
	default:
			break;
	}
	
	P1.m_Pos.Set(&Playfield.m_P1Start);
	P2.m_Pos.Set(&Playfield.m_P2Start);
	P1.m_Score = 0;
	P2.m_Score = 0;
	g_ToServe = 1;

	return 1;
}		

int NullFunc(void* _arg)
{

	return 1;
}

float cc=0;
int FadeIn(void* _arg)
{
	u8 c = R(Playfield.m_Colour);
	int cl;
	if(c<255)
	{
		cc+=0.5f;
		cl = int(cc);
		Playfield.m_Colour = RGB(cl, cl, cl);
		g_Text.colour = RGBA(cl, cl,cl, 255);
	}
	else
	{
		g_pInput = &InputMenu;
		g_pLogic = &LogicMenu;
		g_pRender = &RenderMenu;
	}
	
	RenderMenu(_arg);
	
	
	return 1;
}
//------------------------------------------------------------------------------
// Get Input
//------------------------------------------------------------------------------
int InputGame(void* _arg)
{
	if(g_Core.m_Pad.State & PSP_CTRL_UP)		{	P1.m_Vel.y = -1*P1.m_Speeds.y;	}
	if(g_Core.m_Pad.State & PSP_CTRL_DOWN)		{	P1.m_Vel.y = 	P1.m_Speeds.y;	}
	if(g_Core.m_Pad.State & PSP_CTRL_TRIANGLE)	{	P2.m_Vel.y = -1*P2.m_Speeds.y;	}
	if(g_Core.m_Pad.State & PSP_CTRL_CROSS)		{	P2.m_Vel.y = 	P2.m_Speeds.y;	}
	
	if( ((g_Core.m_Pad.State & PSP_CTRL_LTRIGGER) && (g_ToServe == 1)) ||
		((g_Core.m_Pad.State & PSP_CTRL_RTRIGGER) && (g_ToServe == 2)) )
	{
		g_ToServe = 0;
	}	
		
	//Screen		
	if(g_Core.m_Pad.Press & PSP_CTRL_SELECT)
	{
		timeval s;
		timezone z;
		sceKernelLibcGettimeofday(&s, &z);
		
		char buf[64];
		sprintf(buf, "../../PHOTO/Pong+ [%ld].png", s.tv_sec);
		saveImage(buf, getVramDisplayBuffer(), SCREEN_WIDTH, SCREEN_HEIGHT, PSP_LINE_SIZE, 0);
	}
	
	if(g_Core.m_Pad.Press & PSP_CTRL_START)
	{
		g_pInput = &InputPaused;
		g_pLogic = &LogicPaused;
		g_pRender = &RenderPaused;
		g_Paused = 1;
	}
	
	return 1;
}

//------------------------------------------------------------------------------
// Think
//------------------------------------------------------------------------------
int LogicGame(void* _arg)
{
	// Update	
	P1.Update(Playfield, Ball.m_Pos.y);
	P2.Update(Playfield, Ball.m_Pos.y);
	
	//Waiting for serve?
	switch(g_ToServe)
	{
	case 0:
		Ball.Update(Playfield, P1, P2);
		break;
	case 1:
			{
			Ball.m_Pos.x = P1.m_Pos.x + P1.m_HScale.x + Ball.m_HScale.x + 8;
			Ball.m_Pos.y = P1.m_Pos.y;
			Ball.m_Vel.x = Ball.m_Speeds.x;
			Ball.m_Vel.y = (P1.m_Pos.y / SCREEN_HEIGHT) - 0.5f;
		}
		break;
	case 2:
			{
			Ball.m_Pos.x = P2.m_Pos.x - P1.m_HScale.x - Ball.m_HScale.x - 8;
			Ball.m_Pos.y = P2.m_Pos.y;
			Ball.m_Vel.x = -1 * Ball.m_Speeds.x;
			Ball.m_Vel.y = (P2.m_Pos.y / SCREEN_HEIGHT) - 0.5f;
		}
		break;
	}
	
	
	//Win state
	if((P1.m_Score == WinAt) || (P2.m_Score == WinAt))
	{
		if(P1.m_Score > P2.m_Score)
			P1.m_ScoreGamesWon++;
		else
			P2.m_ScoreGamesWon++;
			
		g_pInput = &InputGameWin;
		g_pLogic = &LogicGameWin;
		g_pRender = &RenderGameWin;
	}
	return 1;		
}

//------------------------------------------------------------------------------
// Draw
//------------------------------------------------------------------------------
int RenderGame(void* _arg)
{
	u64 tp = 0;
	if(_arg)
		tp = *(u64*)_arg;
	
		
	//Render scene
	Playfield.Render();
	P1.Render();
	P2.Render();
	Ball.Render();
					
	//Render FPS
	if((g_Core.m_Pad.State & PSP_CTRL_LTRIGGER)&&(g_Core.m_Pad.State & PSP_CTRL_RTRIGGER))
	{
		float curr_fps = g_Core.m_Timer.GetFPS();
		g_Text.Print(32, 16, "fps: %d.%03d (%dMB/s)",(int)curr_fps,(int)((curr_fps-(int)curr_fps) * 1000.0f),(((int)curr_fps * SCREEN_WIDTH * SCREEN_HEIGHT * 2)/(1024*1024)));
	}
	
	//Render text
	g_Text.PrintVTB(8, 2, 							   "^ Player 1\t\t\t  %02d : %02d",P1.m_Score, P2.m_Score);
	g_Text.PrintVBT(SCREEN_WIDTH-16, SCREEN_HEIGHT-10, "_ Player 2\t\t\t  %02d : %02d",P2.m_Score, P1.m_Score);
	
	return 1;
}

//------------------------------------------------------------------------------
// Main
//------------------------------------------------------------------------------
int main(void)
{
	void *vp = 0;
	
	g_pInput = &NullFunc;
	g_pLogic = &NullFunc;
	g_pRender = &FadeIn;

	// Initialise everything
	Initialise(vp);
	Playfield.m_Colour = RGB(0, 0, 0);		
	
	// Game Loop	
	while(!done)
	{
		u64 timePassed = g_Core.Update();
		void *tp = (void*)&timePassed;

		g_pInput(vp);
		g_pLogic(tp);
		
		// Render scene
		
		g_Core.StartFrame();
		g_pRender(tp);
		g_Core.EndFrame();
		
		
	}

	g_Core.Shutdown();
	return 0;
}
