
#include <pspgu.h>
#include <psptypes.h>
#include <pspkernel.h>

#include <string.h>
#include <stdio.h>	//for vsprintf
#include <stdarg.h>
#include "text.h"
#include "../defines.h"

#define STRING_MAX_LENGTH (256)

typedef struct
{
	unsigned short u, v;
	unsigned int c;
	short x, y, z;
} Vertex;



int CText::Init(char* _filename)
{
	w = 8;
	h = 8;	
	u = 0;
	v = 0;	
	colour = RGBA(255, 255, 255, 255);
	lineSpace = 1;
	
	m_Img = loadImage(_filename);
	
	if (!m_Img)
	{
		printf("failed to load text\n");
		return 0;
	}
	
	return 1;
}

int CText::Print(int _x, int _y, char* _string, ...)
{
	if (_string==NULL) return 0;								// Protect against null strings
	
	va_list			VariableList;								// Variable list
	char			sVarBuffer[STRING_MAX_LENGTH];				// string + substituted variables

	//split for variables
	va_start(VariableList, _string);							// Strip out variables from function call
	vsprintf(sVarBuffer, _string, VariableList);				// Format variables into a string
	va_end(VariableList);	


	int	px = _x,
		py = _y,
		length,
		i,
		num;
	
	length = strlen(sVarBuffer);
	Vertex* vertices;
	
	// Init gu
	sceKernelDcacheWritebackInvalidateAll();
	guStart();

	// copy texture to vram
	sceGuTexMode(GU_PSM_8888,0,0,0); // 16-bit RGBA
	sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA); 
	sceGuTexImage(0, m_Img->textureWidth, m_Img->textureHeight, m_Img->textureWidth, (void*) m_Img->data);

	// Init texture in vram
	float tu = 1.0f / ((float)m_Img->textureWidth);
	float tv = 1.0f / ((float)m_Img->textureHeight);
	sceGuTexScale(tu, tv);
	
	// Draw text
	for (i = 0; i<length; i++)
	{
		switch(sVarBuffer[i]) 
		{
		case '\n':
			{
				py+=h+lineSpace;
				px=_x;
				continue;
			}
			break;
			
		case '\t':
			{
				px+=4*w;
				continue;
			}
			break;
			
		default:
			px += w;		
		}
			
		num = sVarBuffer[i]-32;
		u = (num%8)*8;
		v = (num/8)*8;
		
		// Get 2 new vertices (tl/br)
		vertices = (Vertex*) sceGuGetMemory(2 * sizeof(Vertex));
		
		// Fill vertices with data
		vertices[0].x = px;
		vertices[0].y = py;
		vertices[0].u = u;
		vertices[0].v = v;	
		vertices[0].z = 0;
		vertices[0].c = colour;
								
		vertices[1].x = px + w;
		vertices[1].y = py + h;
		vertices[1].u = u + w;
		vertices[1].v = v + h;		
		vertices[1].z = 0;
		vertices[1].c = colour;
		
		sceGuDrawArray(GU_SPRITES, GU_TEXTURE_16BIT | GU_COLOR_8888 | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 2, 0, vertices);

		//blitAlphaImageToScreen(u ,v ,w , h, m_Img, px, py);
	}
	
	//Release gu
	sceGuFinish();
	sceGuSync(0, 0);
	return 1;
}


int CText::PrintUD(int _x, int _y, char* _string, ...)
{
	if (_string==NULL) return 0;								// Protect against null strings
	
	va_list			VariableList;								// Variable list
	char			sVarBuffer[STRING_MAX_LENGTH];				// string + substituted variables

	//split for variables
	va_start(VariableList, _string);							// Strip out variables from function call
	vsprintf(sVarBuffer, _string, VariableList);				// Format variables into a string
	va_end(VariableList);	


	int	px = _x,
		py = _y,
		length,
		i,
		num;
	length = strlen(sVarBuffer);
	Vertex* vertices;
	
	// Init gu
	sceKernelDcacheWritebackInvalidateAll();
	guStart();

	// copy texture to vram
	sceGuTexMode(GU_PSM_8888,0,0,0); // 16-bit RGBA
	sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA); 
	sceGuTexImage(0, m_Img->textureWidth, m_Img->textureHeight, m_Img->textureWidth, (void*) m_Img->data);

	// Init texture in vram
	float tu = 1.0f / ((float)m_Img->textureWidth);
	float tv = 1.0f / ((float)m_Img->textureHeight);
	sceGuTexScale(tu, tv);
	
	// Draw text
	for (i = 0; i<length; i++)
	{
		switch(sVarBuffer[i]) 
		{
		case '\n':
			{
				py-=(h+lineSpace);
				px=_x;
				continue;
			}
			break;
			
		case '\t':
			{
				px-=4*w;
				continue;
			}
			break;
			
		default:
			px -= w;		
		}
			
		num = sVarBuffer[i]-32;
		u = (num%8)*8;
		v = (num/8)*8;
		
		// Get 2 new vertices (tl/br)
		vertices = (Vertex*) sceGuGetMemory(2 * sizeof(Vertex));
		
		// Fill vertices with data
		vertices[0].x = px;
		vertices[0].y = py;
		vertices[0].u = u+w;
		vertices[0].v = v+h;	
		vertices[0].z = 0;
		vertices[0].c = colour;
								
		vertices[1].x = px + w;
		vertices[1].y = py + h;
		vertices[1].u = u;
		vertices[1].v = v;			
		vertices[1].z = 0;	
		vertices[1].c = colour;
		
		sceGuDrawArray(GU_SPRITES, GU_TEXTURE_16BIT | GU_COLOR_8888 | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 2, 0, vertices);
	}
	
	//Release gu
	sceGuFinish();
	sceGuSync(0, 0);
	return 1;
}


int CText::PrintVTB(int _x, int _y, char* _string, ...)
{
	if (_string==NULL) return 0;								// Protect against null strings
	
	va_list			VariableList;								// Variable list
	char			sVarBuffer[STRING_MAX_LENGTH];				// string + substituted variables

	//split for variables
	va_start(VariableList, _string);							// Strip out variables from function call
	vsprintf(sVarBuffer, _string, VariableList);				// Format variables into a string
	va_end(VariableList);	


	int	px = _x,
		py = _y,
		length,
		i,
		num;
	
	length = strlen(sVarBuffer);
	Vertex* vertices;
	
	// Init gu
	sceKernelDcacheWritebackInvalidateAll();
	guStart();

	// copy texture to vram
	sceGuTexMode(GU_PSM_8888,0,0,0); // 16-bit RGBA
	sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA); 
	sceGuTexImage(0, m_Img->textureWidth, m_Img->textureHeight, m_Img->textureWidth, (void*) m_Img->data);

	// Init texture in vram
	float tu = 1.0f / ((float)m_Img->textureWidth);
	float tv = 1.0f / ((float)m_Img->textureHeight);
	sceGuTexScale(tu, tv);
	
	// Draw text
	for (i = 0; i<length; i++)
	{
		switch(sVarBuffer[i]) 
		{
		case '\n':
			{
				px-=(h+lineSpace);
				py=_y;
				continue;
			}
			break;
			
		case '\t':
			{
				py+=4*w;
				continue;
			}
			break;
			
		default:
			py += w;		
		}
			
		num = sVarBuffer[i]-32;
		u = (num%8)*8;
		v = (num/8)*8;
		
		// Get 4 new vertices (tl/br)
		vertices = (Vertex*) sceGuGetMemory(4 * sizeof(Vertex));
		
		// Fill vertices with data
		vertices[0].x = px;
		vertices[0].y = py;
		vertices[0].u = u;
		vertices[0].v = v + h;	
		vertices[0].z = 0;
		vertices[0].c = colour;
								
		vertices[1].x = px + w;
		vertices[1].y = py;
		vertices[1].u = u;
		vertices[1].v = v;		
		vertices[1].z = 0;	
		vertices[1].c = colour;
		
		vertices[2].x = px + w;
		vertices[2].y = py + h;
		vertices[2].u = u + w;
		vertices[2].v = v;		
		vertices[2].z = 0;	
		vertices[2].c = colour;
		
		vertices[3].x = px;
		vertices[3].y = py + h;
		vertices[3].u = u + w;
		vertices[3].v = v + h;		
		vertices[3].z = 0;
		vertices[3].c = colour;
		
		sceGuDrawArray(GU_TRIANGLE_FAN, GU_TEXTURE_16BIT | GU_COLOR_8888 | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 4, 0, vertices);

		//blitAlphaImageToScreen(u ,v ,w , h, m_Img, px, py);
	}
	
	//Release gu
	sceGuFinish();
	sceGuSync(0, 0);
	return 1;
}


int CText::PrintVBT(int _x, int _y, char* _string, ...)
{
	if (_string==NULL) return 0;								// Protect against null strings
	
	va_list			VariableList;								// Variable list
	char			sVarBuffer[STRING_MAX_LENGTH];				// string + substituted variables

	//split for variables
	va_start(VariableList, _string);							// Strip out variables from function call
	vsprintf(sVarBuffer, _string, VariableList);				// Format variables into a string
	va_end(VariableList);	


	int	px = _x,
		py = _y,
		length,
		i,
		num;
	
	length = strlen(sVarBuffer);
	Vertex* vertices;
	
	// Init gu
	sceKernelDcacheWritebackInvalidateAll();
	guStart();

	// copy texture to vram
	sceGuTexMode(GU_PSM_8888,0,0,0); // 16-bit RGBA
	sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA); 
	sceGuTexImage(0, m_Img->textureWidth, m_Img->textureHeight, m_Img->textureWidth, (void*) m_Img->data);

	// Init texture in vram
	float tu = 1.0f / ((float)m_Img->textureWidth);
	float tv = 1.0f / ((float)m_Img->textureHeight);
	sceGuTexScale(tu, tv);
	
	// Draw text
	for (i = 0; i<length; i++)
	{
		switch(sVarBuffer[i]) 
		{
		case '\n':
			{
				px+=h+lineSpace;
				py=_y;
				continue;
			}
			break;
			
		case '\t':
			{
				py-=4*w;
				continue;
			}
			break;
			
		default:
			py -= w;		
		}
			
		num = sVarBuffer[i]-32;
		u = (num%8)*8;
		v = (num/8)*8;
		
		// Get 4 new vertices (tl/br)
		vertices = (Vertex*) sceGuGetMemory(4 * sizeof(Vertex));
		
		// Fill vertices with data
		vertices[0].x = px;
		vertices[0].y = py;
		vertices[0].u = u + w;
		vertices[0].v = v;	
		vertices[0].z = 0;
		vertices[0].c = colour;
								
		vertices[1].x = px + w;
		vertices[1].y = py;
		vertices[1].u = u + w;
		vertices[1].v = v + h;		
		vertices[1].z = 0;	
		vertices[1].c = colour;
		
		vertices[2].x = px + w;
		vertices[2].y = py + h;
		vertices[2].u = u;
		vertices[2].v = v + h;		
		vertices[2].z = 0;	
		vertices[2].c = colour;
		
		vertices[3].x = px;
		vertices[3].y = py + h;
		vertices[3].u = u;
		vertices[3].v = v;		
		vertices[3].z = 0;
		vertices[3].c = colour;
		
		sceGuDrawArray(GU_TRIANGLE_FAN, GU_TEXTURE_16BIT | GU_COLOR_8888 | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 4, 0, vertices);

		//blitAlphaImageToScreen(u ,v ,w , h, m_Img, px, py);
	}
	
	//Release gu
	sceGuFinish();
	sceGuSync(0, 0);
	return 1;
}
